# /usr/bin/env python3
import json
import os
from typing import Any, cast

import requests
from dotenv import load_dotenv

from ginear.utils import DOTFILE_PATH, clear_env_key, switch_branch

load_dotenv(dotenv_path=DOTFILE_PATH)


TEAM_ID = os.environ.get("TEAM_ID")

PROJECT_ID = os.environ.get("PROJECT_ID")

USER_ID = os.environ.get("USER_ID")

INITIAL_STATE_ID = os.environ.get("INITIAL_STATE_ID")


def get_user_id() -> dict[str, Any]:
    query = """
    query Me {
        viewer {
            id
            name
            email
            teams {
                nodes {
                    id
                    name
                }
            }
        }
    }
    """

    request_data = {"query": query}

    result = call_linear_api(request_data)
    return cast(dict[str, Any], result["viewer"])


def get_team_ids() -> list[dict[str, Any]]:
    query = """
    query {
        teams(first: 250) {
            nodes {
                id
                name
            }
        }
    }
    """

    request_data = {"query": query}

    result = call_linear_api(request_data)
    return cast(list[dict[str, Any]], result["teams"]["nodes"])


def get_project_ids_for_team(team_id: str) -> list[dict[str, Any]]:
    query = """
    query GetProjectsInTeam($teamId: String!) {
        team(id: $teamId) {
            id
            name
            projects {
                nodes {
                    id
                    name
                }
            }
        }
    }
    """

    variables = {
        "teamId": team_id,
    }

    request_data = {"query": query, "variables": variables}

    result = call_linear_api(request_data)
    return cast(list[dict[str, Any]], result["team"]["projects"]["nodes"])


def get_state_ids_for_team(team_id: str) -> list[dict[str, Any]]:
    query = """
    query GetStatesAndPrioritiesInProject($teamId: String!) {
        team(id: $teamId) {
            id
            name
            states {
                nodes {
                    id
                    name
                }
            }
        }
    }
    """

    variables = {
        "teamId": team_id,
    }

    request_data = {"query": query, "variables": variables}

    result = call_linear_api(request_data)
    return cast(list[dict[str, Any]], result["team"]["states"]["nodes"])


def get_issues() -> list[dict[str, Any]]:
    query = """
    query ($teamId: String!) {
        team (id: $teamId) {
            issues(first:250) {
                edges {
                    node {
                        id
                        title
                        branchName
                        creator {
                            name
                        }
                    }
                }
                pageInfo {
                    hasNextPage
                    endCursor
                }
            }
        }
    }
    """

    variables = {
        "teamId": TEAM_ID,
    }

    request_data = {"query": query, "variables": variables}
    result = call_linear_api(request_data)

    return [edge["node"] for edge in result["team"]["issues"]["edges"]]


def create_issue(title: str, description: str) -> None:
    mutation = """
    mutation IssueCreate($title: String!, $description: String!, $teamId: String!, $assigneeId: String!, $stateId: String!, $projectId: String!) {
    issueCreate(
        input: {
            title: $title
            description: $description
            teamId: $teamId
            assigneeId: $assigneeId
            stateId: $stateId
            projectId: $projectId
        }
    ) {
        success
            issue {
                id
                title
                branchName
            }
        }
    }
    """

    mutation_variables = {
        "title": title,
        "description": description,
        "teamId": TEAM_ID,
        "assigneeId": USER_ID,
        "stateId": INITIAL_STATE_ID,
        "projectId": PROJECT_ID,
    }

    request_data = {"query": mutation, "variables": mutation_variables}
    result = call_linear_api(request_data)
    issue_create_response = result["issueCreate"]

    if issue_create_response["success"]:
        issue = issue_create_response["issue"]
        print(
            f"Issue created successfully. ID: {issue['id']}, Title: {issue['title']}, branch: {issue['branchName']}"
        )
        switch_branch(issue["branchName"])
    else:
        print("Issue creation failed.")


def call_linear_api(request_data: dict[str, Any]) -> dict[str, Any]:
    LINEAR_API_TOKEN = os.environ.get("LINEAR_API_TOKEN")

    if not LINEAR_API_TOKEN:
        load_dotenv(dotenv_path=DOTFILE_PATH)
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"{LINEAR_API_TOKEN}",
    }

    api_endpoint = "https://api.linear.app/graphql"
    print(request_data)

    response = requests.post(
        api_endpoint, data=json.dumps(request_data), headers=headers
    )

    response_data = response.json()

    if "errors" in response_data:
        print(f"Error calling {api_endpoint}")
        print(response_data["errors"])
        try:
            if (
                response_data["errors"][0]["extensions"]["code"]
                == "AUTHENTICATION_ERROR"
            ):
                clear_env_key("LINEAR_API_TOKEN")
            raise ValueError("Invalid API token")
        except Exception:
            pass

        raise Exception("Unknown API error")

    result = response_data["data"]
    return cast(dict[str, Any], result)



# /usr/bin/env python3
from pathlib import Path

from dotenv import set_key, unset_key

DOTFILE_PATH = Path.home() / ".ginear"


def write_to_env(key: str, value: str) -> None:
    set_key(DOTFILE_PATH, key, value)


def clear_env_key(key: str) -> None:
    unset_key(DOTFILE_PATH, key)


def switch_branch(branch_name: str) -> None:
    import subprocess

    try:
        subprocess.run(["git", "switch", "-c", branch_name], check=True)
        print(f"Created and switched to branch '{branch_name}' successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error: {e}")



# /usr/bin/env python3
import os

import typer
from dotenv import load_dotenv

from ginear.queries import (
    create_issue,
    get_issues,
    get_project_ids_for_team,
    get_state_ids_for_team,
    get_team_ids,
    get_user_id,
)
from ginear.utils import DOTFILE_PATH, switch_branch, write_to_env

load_dotenv(dotenv_path=DOTFILE_PATH)
app = typer.Typer()

LINEAR_API_TOKEN = os.environ.get("LINEAR_API_TOKEN")

TEAM_ID = os.environ.get("TEAM_ID")

PROJECT_ID = os.environ.get("PROJECT_ID")

USER_ID = os.environ.get("USER_ID")

INITIAL_STATE_ID = os.environ.get("INITIAL_STATE_ID")


def create_issue_prompt() -> None:
    title = typer.prompt("Title")
    description = typer.prompt("Description")
    create_issue(title, description)


def attach_issue_prompt() -> None:
    from pyfzf.pyfzf import FzfPrompt

    issues = get_issues()
    fzf = FzfPrompt()
    selected_list = fzf.prompt(
        [
            "> Create new issue",
            *[
                f"[{issue['creator']['name'][:10]}] – {issue['title']}"
                for issue in issues
            ],
        ]
    )
    if selected_list:
        selected = selected_list[0]
        if selected == "> SKIP":
            print("Skipping ticket")
            return

        if selected == "> Create new issue":
            create_issue_prompt()
            return

        issue = next(
            issue for issue in issues if issue["title"] == selected.split(" – ")[1]
        )
        branch_name = issue["branchName"]
        switch_branch(branch_name)

    print("Selected:", selected)


def set_project(team_id: str) -> None:
    from pyfzf.pyfzf import FzfPrompt

    project_ids = get_project_ids_for_team(team_id)
    fzf = FzfPrompt()
    selected_list = fzf.prompt(
        [
            *[
                f"[{project_id['name']}] – {project_id['id']}"
                for project_id in project_ids
            ],
        ]
    )
    if not selected_list:
        raise ValueError("Missing projects")

    project_id = selected_list[0].split(" – ")[1]
    if not project_id:
        raise ValueError("No project id")
    write_to_env("PROJECT_ID", project_id)


def set_state(team_id: str) -> None:
    from pyfzf.pyfzf import FzfPrompt

    state_ids = get_state_ids_for_team(team_id)
    fzf = FzfPrompt()
    selected_list = fzf.prompt(
        [
            *[
                f"[{initial_issue_state['name']}] – {initial_issue_state['id']}"
                for initial_issue_state in state_ids
            ],
        ]
    )
    if not selected_list:
        raise ValueError("Missing states")
    initial_issue_state = selected_list[0].split(" – ")[1]
    if not initial_issue_state:
        raise ValueError("No initial issue state")
    write_to_env("INITIAL_STATE_ID", initial_issue_state)


def run_onboarding() -> None:
    from pyfzf.pyfzf import FzfPrompt

    if not LINEAR_API_TOKEN:
        import webbrowser

        webbrowser.open("https://linear.app/settings/api")
        linear_api_token = typer.prompt(
            "Insert Personal API Key (https://linear.app/settings/api): "
        )
        if not linear_api_token:
            raise ValueError("Invalid API token")

        write_to_env("LINEAR_API_TOKEN", linear_api_token)
        load_dotenv(dotenv_path=DOTFILE_PATH)

    user_id = USER_ID
    if not user_id:
        user = get_user_id()
        user_id = user["id"]
        if not user_id:
            raise ValueError("No user id")
        write_to_env("USER_ID", user_id)

    team_id = TEAM_ID
    if not team_id:
        team_ids = get_team_ids()
        fzf = FzfPrompt()
        selected_list = fzf.prompt(
            [
                *[f"[{team_id['name']}] – {team_id['id']}" for team_id in team_ids],
            ]
        )
        if not selected_list:
            raise ValueError("Missing team ids")

        team_id = selected_list[0].split(" – ")[1]
        if not team_id:
            raise ValueError("No team id")
        write_to_env("TEAM_ID", team_id)

    project_id = PROJECT_ID
    if not project_id:
        set_project(team_id=team_id)

    initial_issue_state = INITIAL_STATE_ID
    if not initial_issue_state:
        set_state(team_id=team_id)

    print("🍸 Onboarding success 🍸")


@app.command()
def project() -> None:
    if not TEAM_ID:
        raise ValueError("Missing team_id")

    set_project(TEAM_ID)


@app.callback(invoke_without_command=True)
def main(ctx: typer.Context) -> None:
    if ctx.invoked_subcommand:
        return

    if LINEAR_API_TOKEN and TEAM_ID and PROJECT_ID and USER_ID and INITIAL_STATE_ID:
        print("🍸 Ginear ticket 🍸")
        attach_issue_prompt()
    else:
        print("🍸 Initializing ginear 🍸")
        run_onboarding()


def run() -> None:
    app()


if __name__ == "__main__":
    app()
